@@ -5,11 +5,8 @@ imported after the start of the test run, excluding modules that match
 testMatch. If you want to include those modules too, use the ``--cover-tests``
 switch, or set the NOSE_COVER_TESTS environment variable to a true value. To
 restrict the coverage report to modules from a particular package or packages,
-use the ``--cover-package`` switch or the NOSE_COVER_PACKAGE environment
-variable.
-
-.. _coverage: http://www.nedbatchelder.com/code/modules/coverage.html
-"""
+use the --cover-package switch or the NOSE_COVER_PACKAGE environment variable.
+.. _coverage: https://coverage.readthedocs.io/ """
 import logging
 import re
 import sys
@@ -21,9 +18,7 @@ log = logging.getLogger(__name__)
 
 
 class Coverage(Plugin):
-    """
-    Activate a coverage report using Ned Batchelder's coverage module.
-    """
+    """Activate a coverage report using Ned Batchelder's coverage module."""
     coverTests = False
     coverPackages = None
     coverInstance = None
@@ -33,74 +28,66 @@ class Coverage(Plugin):
     status = {}
 
     def options(self, parser, env):
-        """
-        Add options to command line.
-        """
+        """Add options to the command line."""
         super(Coverage, self).options(parser, env)
-        parser.add_option("--cover-package", action="append",
-                          default=env.get('NOSE_COVER_PACKAGE'),
-                          metavar="PACKAGE",
-                          dest="cover_packages",
-                          help="Restrict coverage output to selected packages "
-                          "[NOSE_COVER_PACKAGE]")
-        parser.add_option("--cover-erase", action="store_true",
-                          default=env.get('NOSE_COVER_ERASE'),
-                          dest="cover_erase",
-                          help="Erase previously collected coverage "
-                          "statistics before run")
-        parser.add_option("--cover-tests", action="store_true",
-                          dest="cover_tests",
-                          default=env.get('NOSE_COVER_TESTS'),
-                          help="Include test modules in coverage report "
-                          "[NOSE_COVER_TESTS]")
-        parser.add_option("--cover-min-percentage", action="store",
-                          dest="cover_min_percentage",
-                          default=env.get('NOSE_COVER_MIN_PERCENTAGE'),
-                          help="Minimum percentage of coverage for tests "
-                          "to pass [NOSE_COVER_MIN_PERCENTAGE]")
-        parser.add_option("--cover-inclusive", action="store_true",
-                          dest="cover_inclusive",
-                          default=env.get('NOSE_COVER_INCLUSIVE'),
-                          help="Include all python files under working "
-                          "directory in coverage report.  Useful for "
-                          "discovering holes in test coverage if not all "
-                          "files are imported by the test suite. "
-                          "[NOSE_COVER_INCLUSIVE]")
-        parser.add_option("--cover-html", action="store_true",
-                          default=env.get('NOSE_COVER_HTML'),
-                          dest='cover_html',
-                          help="Produce HTML coverage information")
-        parser.add_option('--cover-html-dir', action='store',
-                          default=env.get('NOSE_COVER_HTML_DIR', 'cover'),
-                          dest='cover_html_dir',
-                          metavar='DIR',
-                          help='Produce HTML coverage information in dir')
-        parser.add_option("--cover-branches", action="store_true",
-                          default=env.get('NOSE_COVER_BRANCHES'),
-                          dest="cover_branches",
-                          help="Include branch coverage in coverage report "
-                          "[NOSE_COVER_BRANCHES]")
-        parser.add_option("--cover-xml", action="store_true",
-                          default=env.get('NOSE_COVER_XML'),
-                          dest="cover_xml",
-                          help="Produce XML coverage information")
-        parser.add_option("--cover-xml-file", action="store",
-                          default=env.get('NOSE_COVER_XML_FILE', 'coverage.xml'),
-                          dest="cover_xml_file",
-                          metavar="FILE",
-                          help="Produce XML coverage information in file")
+        parser.add_option(
+            "--cover-package", action="append",
+            default=env.get('NOSE_COVER_PACKAGE'),
+            metavar="PACKAGE", dest="cover_packages",
+            help="Restrict coverage output to selected packages "
+            "[NOSE_COVER_PACKAGE]")
+        parser.add_option(
+            "--cover-erase", action="store_true",
+            default=env.get('NOSE_COVER_ERASE'), dest="cover_erase",
+            help="Erase previously collected coverage statistics before run")
+        parser.add_option(
+            "--cover-tests", action="store_true", dest="cover_tests",
+            default=env.get('NOSE_COVER_TESTS'),
+            help="Include test modules in coverage report [NOSE_COVER_TESTS]")
+        parser.add_option(
+            "--cover-min-percentage", action="store",
+            dest="cover_min_percentage",
+            default=env.get('NOSE_COVER_MIN_PERCENTAGE'),
+            help="Minimum percentage of coverage for tests "
+            "to pass [NOSE_COVER_MIN_PERCENTAGE]")
+        parser.add_option(
+            "--cover-inclusive", action="store_true", dest="cover_inclusive",
+            default=env.get('NOSE_COVER_INCLUSIVE'),
+            help="Include all python files under working "
+            "directory in coverage report. Useful for "
+            "discovering holes in test coverage if not all "
+            "files are imported by the test suite. [NOSE_COVER_INCLUSIVE]")
+        parser.add_option(
+            "--cover-html", action="store_true",
+            default=env.get('NOSE_COVER_HTML'), dest='cover_html',
+            help="Produce HTML coverage information")
+        parser.add_option(
+            '--cover-html-dir', action='store',
+            default=env.get('NOSE_COVER_HTML_DIR', 'cover'),
+            dest='cover_html_dir', metavar='DIR',
+            help='Produce HTML coverage information in dir')
+        parser.add_option(
+            "--cover-branches", action="store_true",
+            default=env.get('NOSE_COVER_BRANCHES'), dest="cover_branches",
+            help="Include branch coverage in coverage report "
+            "[NOSE_COVER_BRANCHES]")
+        parser.add_option(
+            "--cover-xml", action="store_true",
+            default=env.get('NOSE_COVER_XML'), dest="cover_xml",
+            help="Produce XML coverage information")
+        parser.add_option(
+            "--cover-xml-file", action="store",
+            default=env.get('NOSE_COVER_XML_FILE', 'coverage.xml'),
+            dest="cover_xml_file", metavar="FILE",
+            help="Produce XML coverage information in file")
 
     def configure(self, options, conf):
-        """
-        Configure plugin.
-        """
+        """Configure plugin."""
         try:
             self.status.pop('active')
         except KeyError:
             pass
         super(Coverage, self).configure(options, conf)
-        if conf.worker:
-            return
         if self.enabled:
             try:
                 import coverage
@@ -133,44 +120,59 @@ class Coverage(Plugin):
         self.coverBranches = options.cover_branches
         self.coverXmlFile = None
         if options.cover_min_percentage:
-            self.coverMinPercentage = int(options.cover_min_percentage.rstrip('%'))
+            self.coverMinPercentage = int(
+                options.cover_min_percentage.rstrip('%')
+            )
         if options.cover_xml:
             self.coverXmlFile = options.cover_xml_file
             log.debug('Will put XML coverage report in %s', self.coverXmlFile)
         if self.enabled:
             self.status['active'] = True
-            self.coverInstance = coverage.coverage(auto_data=False,
-                branch=self.coverBranches, data_suffix=None,
-                source=self.coverPackages)
+            self.coverInstance = coverage.coverage(
+                auto_data=False,
+                branch=self.coverBranches, data_suffix=conf.worker,
+                source=self.coverPackages
+            )
+            self.coverInstance._warn_no_data = False
+            self.coverInstance.is_worker = conf.worker
+            self.coverInstance.exclude(
+                '#pragma[: ]+[nN][oO] [cC][oO][vV][eE][rR]'
+            )
+            log.debug("Coverage begin")
+            self.skipModules = list(sys.modules.keys())[:]
+            if self.coverErase:
+                log.debug("Clearing previously collected coverage statistics")
+                self.coverInstance.combine()
+                self.coverInstance.erase()
+            if not self.coverInstance.is_worker:
+                self.coverInstance.load()
+                self.coverInstance.start()
 
-    def begin(self):
-        """
-        Begin recording coverage information.
-        """
-        log.debug("Coverage begin")
-        self.skipModules = list(sys.modules.keys())[:]
-        if self.coverErase:
-            log.debug("Clearing previously collected coverage statistics")
-            self.coverInstance.combine()
-            self.coverInstance.erase()
-        self.coverInstance.exclude('#pragma[: ]+[nN][oO] [cC][oO][vV][eE][rR]')
-        self.coverInstance.load()
-        self.coverInstance.start()
+    def beforeTest(self, *args, **kwargs):
+        """Begin recording coverage information."""
+        if self.coverInstance.is_worker:
+            self.coverInstance.load()
+            self.coverInstance.start()
+
+    def afterTest(self, *args, **kwargs):
+        """Stop recording coverage information."""
+        if self.coverInstance.is_worker:
+            self.coverInstance.stop()
+            self.coverInstance.save()
 
     def report(self, stream):
-        """
-        Output code coverage report.
-        """
+        """Output code coverage report."""
         log.debug("Coverage report")
         self.coverInstance.stop()
         self.coverInstance.combine()
         self.coverInstance.save()
-        modules = [module
-                    for name, module in list(sys.modules.items())
-                    if self.wantModuleCoverage(name, module)]
+        modules = [
+            module
+            for name, module in list(sys.modules.items())
+            if self.wantModuleCoverage(name, module)
+        ]
         log.debug("Coverage report will cover modules: %s", modules)
         self.coverInstance.report(modules, file=stream)
-
         import coverage
         if self.coverHtmlDir:
             log.debug("Generating HTML coverage report")
@@ -178,28 +180,22 @@ class Coverage(Plugin):
                 self.coverInstance.html_report(modules, self.coverHtmlDir)
             except coverage.misc.CoverageException as e:
                 log.warning("Failed to generate HTML report: %s" % str(e))
-
         if self.coverXmlFile:
             log.debug("Generating XML coverage report")
             try:
                 self.coverInstance.xml_report(modules, self.coverXmlFile)
             except coverage.misc.CoverageException as e:
                 log.warning("Failed to generate XML report: %s" % str(e))
-
-        # make sure we have minimum required coverage
         if self.coverMinPercentage:
             f = io.StringIO()
             self.coverInstance.report(modules, file=f)
-
             multiPackageRe = (r'-------\s\w+\s+\d+\s+\d+(?:\s+\d+\s+\d+)?'
                               r'\s+(\d+)%\s+\d*\s{0,1}$')
             singlePackageRe = (r'-------\s[\w./]+\s+\d+\s+\d+(?:\s+\d+\s+\d+)?'
                                r'\s+(\d+)%(?:\s+[-\d, ]+)\s{0,1}$')
-
             m = re.search(multiPackageRe, f.getvalue())
             if m is None:
                 m = re.search(singlePackageRe, f.getvalue())
-
             if m:
                 percentage = int(m.groups()[0])
                 if percentage < self.coverMinPercentage:
@@ -210,7 +206,6 @@ class Coverage(Plugin):
                 log.error("No total percentage was found in coverage output, "
                           "something went wrong.")
 
-
     def wantModuleCoverage(self, name, module):
         if not hasattr(module, '__file__'):
             log.debug("no coverage of %s: no __file__", name)
@@ -233,15 +228,9 @@ class Coverage(Plugin):
         if self.conf.testMatch.search(name) and not self.coverTests:
             log.debug("no coverage for %s: is a test", name)
             return False
-        # accept any package that passed the previous tests, unless
-        # coverPackages is on -- in that case, if we wanted this
-        # module, we would have already returned True
         return not self.coverPackages
 
     def wantFile(self, file, package=None):
-        """If inclusive coverage enabled, return true for all source files
-        in wanted packages.
-        """
         if self.coverInclusive:
             if file.endswith(".py"):
                 if package and self.coverPackages:
