@@ -1,34 +1,26 @@
 """
 This plugin captures logging statements issued during test execution. When an
 error or failure occurs, the captured log messages are attached to the running
-test in the test.capturedLogging attribute, and displayed with the error failure
-output. It is enabled by default but can be turned off with the option
-``--nologcapture``.
-
-You can filter captured logging statements with the ``--logging-filter`` option. 
-If set, it specifies which logger(s) will be captured; loggers that do not match
-will be passed. Example: specifying ``--logging-filter=sqlalchemy,myapp``
+test in the test.capturedLogging attribute, and displayed with the error
+failure output. It is enabled by default but can be turned off with the option
+`--nologcapture`.
+You can filter captured logging statements with the `--logging-filter` option.
+If set, it specifies which logger(s) will be captured; loggers that do not
+match will be passed. Example: specifying `--logging-filter=sqlalchemy,myapp`
 will ensure that only statements logged via sqlalchemy.engine, myapp
 or myapp.foo.bar logger will be logged.
-
 You can remove other installed logging handlers with the
-``--logging-clear-handlers`` option.
-"""
-
+``--logging-clear-handlers`` option."""
 import logging
-from logging import Handler
+import os
 import threading
-
+from logging import Handler
 from nose.plugins.base import Plugin
 from nose.util import anyp, ln, safe_str
 
-try:
-    from io import StringIO
-except ImportError:
-    from io import StringIO
-
 log = logging.getLogger(__name__)
 
+
 class FilterSet(object):
     def __init__(self, filter_components):
         self.inclusive, self.exclusive = self._partition(filter_components)
@@ -47,7 +39,6 @@ class FilterSet(object):
     def allow(self, record):
         """returns whether this record should be printed"""
         if not self:
-            # nothing to filter
             return True
         return self._allow(record) and not self._deny(record)
 
@@ -78,31 +69,36 @@ class MyMemoryHandler(Handler):
         self.setFormatter(fmt)
         self.filterset = FilterSet(filters)
         self.buffer = []
+
     def emit(self, record):
         self.buffer.append(self.format(record))
+
     def flush(self):
-        pass # do nothing
+        pass
+
     def truncate(self):
         self.buffer = []
+
     def filter(self, record):
         if self.filterset.allow(record.name):
             return Handler.filter(self, record)
+        return None
+
     def __getstate__(self):
         state = self.__dict__.copy()
         del state['lock']
         return state
+
     def __setstate__(self, state):
         self.__dict__.update(state)
         self.lock = threading.RLock()
 
 
 class LogCapture(Plugin):
-    """
-    Log capture plugin. Enabled by default. Disable with --nologcapture.
+    """Log capture plugin. Enabled by default. Disable with --nologcapture.
     This plugin captures logging statements issued during test execution,
     appending any output captured to the error or failure output,
-    should the test fail or raise an error.
-    """
+    should the test fail or raise an error."""
     enabled = True
     env_opt = 'NOSE_NOLOGCAPTURE'
     name = 'logcapture'
@@ -113,14 +109,25 @@ class LogCapture(Plugin):
     filters = ['-nose']
 
     def options(self, parser, env):
-        """Register commandline options.
-        """
+        """Register commandline options."""
         parser.add_option(
             "--nologcapture", action="store_false",
-            default=not env.get(self.env_opt), dest="logcapture",
+            default=0, dest="logcapture",
             help="Disable logging capture plugin. "
-                 "Logging configurtion will be left intact."
+                 "Logging configuration will be left intact."
                  " [NOSE_NOLOGCAPTURE]")
+        parser.add_option(
+            "--capture-logs", "--capture_logs", action="store_true",
+            default=0, dest="capture_logs",
+            help="Enable logging capture plugin. "
+                 "Logging configuration will be left intact."
+                 " [NOSE_CAPTURELOGS]")
+        parser.add_option(
+            "--logging-init", action="store_true",
+            default=0, dest="logging_init",
+            help="Initialize standard logging configuration with:\n"
+                 "logging.basicConfig(level)\n"
+                 " [NOSE_LOGINIT]")
         parser.add_option(
             "--logging-format", action="store", dest="logcapture_format",
             default=env.get('NOSE_LOGFORMAT') or self.logformat,
@@ -137,15 +144,15 @@ class LogCapture(Plugin):
                  " [NOSE_LOGDATEFMT]")
         parser.add_option(
             "--logging-filter", action="store", dest="logcapture_filters",
-            default=env.get('NOSE_LOGFILTER'),
-            metavar="FILTER",
+            default=env.get('NOSE_LOGFILTER'), metavar="FILTER",
             help="Specify which statements to filter in/out. "
                  "By default, everything is captured. If the output is too"
                  " verbose,\nuse this option to filter out needless output.\n"
                  "Example: filter=foo will capture statements issued ONLY to\n"
                  " foo or foo.what.ever.sub but not foobar or other logger.\n"
                  "Specify multiple loggers with comma: filter=foo,bar,baz.\n"
-                 "If any logger name is prefixed with a minus, eg filter=-foo,\n"
+                 "If any logger name is prefixed with a minus, eg filter=-foo,"
+                 "\n"
                  "it will be excluded rather than included. Default: "
                  "exclude logging messages from nose itself (-nose)."
                  " [NOSE_LOGFILTER]\n")
@@ -155,21 +162,28 @@ class LogCapture(Plugin):
             help="Clear all other logging handlers")
         parser.add_option(
             "--logging-level", action="store",
-            default='NOTSET', dest="logcapture_level",
+            default='WARNING', dest="logcapture_level",
             help="Set the log level to capture")
 
     def configure(self, options, conf):
-        """Configure plugin.
-        """
+        """Configure plugin."""
         self.conf = conf
-        # Disable if explicitly disabled, or if logging is
-        # configured via logging config file
-        if not options.logcapture or conf.loggingConfig:
-            self.enabled = False
         self.logformat = options.logcapture_format
         self.logdatefmt = options.logcapture_datefmt
         self.clear = options.logcapture_clear
         self.loglevel = options.logcapture_level
+        if not options.logcapture or conf.loggingConfig:
+            self.enabled = False
+        if (
+            "NOSE_CAPTURELOGS" in os.environ
+            and os.environ["NOSE_CAPTURELOGS"] == "1"
+        ) or options.capture_logs:
+            self.enabled = True
+        if (
+            "NOSE_LOGINIT" in os.environ
+            and os.environ["NOSE_LOGINIT"] == "1"
+        ) or options.logging_init:
+            logging.basicConfig(level=self.loglevel)
         if options.logcapture_filters:
             self.filters = options.logcapture_filters.split(',')
 
@@ -184,12 +198,6 @@ class LogCapture(Plugin):
                 if hasattr(logger, "handlers"):
                     for handler in logger.handlers:
                         logger.removeHandler(handler)
-        # make sure there isn't one already
-        # you can't simply use "if self.handler not in root_logger.handlers"
-        # since at least in unit tests this doesn't work --
-        # LogCapture() is instantiated for each test case while root_logger
-        # is module global
-        # so we always add new MyMemoryHandler instance
         for handler in root_logger.handlers[:]:
             if isinstance(handler, MyMemoryHandler):
                 root_logger.handlers.remove(handler)
@@ -199,36 +207,32 @@ class LogCapture(Plugin):
         root_logger.setLevel(getattr(logging, loglevel))
 
     def begin(self):
-        """Set up logging handler before test run begins.
-        """
+        """Set up logging handler before test run begins."""
         self.start()
 
     def start(self):
-        self.handler = MyMemoryHandler(self.logformat, self.logdatefmt,
-                                       self.filters)
+        self.handler = MyMemoryHandler(
+            self.logformat, self.logdatefmt, self.filters
+        )
         self.setupLoghandler()
 
     def end(self):
         pass
 
     def beforeTest(self, test):
-        """Clear buffers and handlers before test.
-        """
+        """Clear buffers and handlers before test."""
         self.setupLoghandler()
 
     def afterTest(self, test):
-        """Clear buffers after test.
-        """
+        """Clear buffers after test."""
         self.handler.truncate()
 
     def formatFailure(self, test, err):
-        """Add captured log messages to failure output.
-        """
+        """Add captured log messages to failure output."""
         return self.formatError(test, err)
 
     def formatError(self, test, err):
-        """Add captured log messages to error output.
-        """
+        """Add captured log messages to error output."""
         # logic flow copied from Capture.formatError
         test.capturedLogging = records = self.formatLogRecords()
         if not records:
@@ -240,6 +244,8 @@ class LogCapture(Plugin):
         return list(map(safe_str, self.handler.buffer))
 
     def addCaptureToErr(self, ev, records):
-        return '\n'.join([safe_str(ev), ln('>> begin captured logging <<')] + \
-                          records + \
-                          [ln('>> end captured logging <<')])
+        return '\n'.join(
+            [safe_str(ev), ln('>> begin captured logging <<')]
+            + records
+            + [ln('>> end captured logging <<')]
+        )
