@@ -1,49 +1,43 @@
-"""Utility functions and classes used by nose internally.
-"""
+"""Utility functions and classes used by nose internally."""
 import inspect
 import itertools
 import logging
+import stat
 import os
 import re
 import sys
 import types
 import unittest
-from nose.pyversion import ClassType, TypeType, isgenerator, ismethod
-
+from nose.pyversion import ClassType, TypeType, isgenerator
 
 log = logging.getLogger('nose')
-
 ident_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_.]*$')
 class_types = (ClassType, TypeType)
-skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)|(?:__pycache__)"
-
-try:
-    set()
-    set = set # make from nose.util import set happy
-except NameError:
-    try:
-        from sets import Set as set
-    except ImportError:
-        pass
+skip_pattern = (
+    r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)|(?:__pycache__)"
+)
+if not hasattr(inspect, "getargspec"):
+    inspect.getargspec = lambda func: inspect.getfullargspec(func)[:4]
 
 
 def ls_tree(dir_path="",
             skip_pattern=skip_pattern,
             indent="|-- ", branch_indent="|   ",
             last_indent="`-- ", last_branch_indent="    "):
-    # TODO: empty directories look like non-directory files
-    return "\n".join(_ls_tree_lines(dir_path, skip_pattern,
-                                    indent, branch_indent,
-                                    last_indent, last_branch_indent))
+    return "\n".join(_ls_tree_lines(
+        dir_path,
+        skip_pattern,
+        indent,
+        branch_indent,
+        last_indent,
+        last_branch_indent
+    ))
 
 
 def _ls_tree_lines(dir_path, skip_pattern,
                    indent, branch_indent, last_indent, last_branch_indent):
     if dir_path == "":
         dir_path = os.getcwd()
-
-    lines = []
-
     names = os.listdir(dir_path)
     names.sort()
     dirs, nondirs = [], []
@@ -54,10 +48,11 @@ def _ls_tree_lines(dir_path, skip_pattern,
             dirs.append(name)
         else:
             nondirs.append(name)
-
     # list non-directories first
-    entries = list(itertools.chain([(name, False) for name in nondirs],
-                                   [(name, True) for name in dirs]))
+    entries = list(itertools.chain(
+        [(name, False) for name in nondirs], [(name, True) for name in dirs]
+    ))
+
     def ls_entry(name, is_dir, ind, branch_ind):
         if not is_dir:
             yield ind + name
@@ -80,26 +75,20 @@ def _ls_tree_lines(dir_path, skip_pattern,
 
 
 def absdir(path):
-    """Return absolute, normalized path to directory, if it exists; None
-    otherwise.
-    """
     if not os.path.isabs(path):
-        path = os.path.normpath(os.path.abspath(os.path.join(os.getcwd(),
-                                                             path)))
+        path = os.path.normpath(
+            os.path.abspath(os.path.join(os.getcwd(), path))
+        )
     if path is None or not os.path.isdir(path):
         return None
     return path
 
 
 def absfile(path, where=None):
-    """Return absolute, normalized path to file (optionally in directory
-    where), or None if the file can't be found either in where or the current
-    working directory.
-    """
     orig = path
     if where is None:
         where = os.getcwd()
-    if isinstance(where, list) or isinstance(where, tuple):
+    if isinstance(where, (list, tuple)):
         for maybe_path in where:
             maybe_abs = absfile(path, maybe_path)
             if maybe_abs is not None:
@@ -109,14 +98,13 @@ def absfile(path, where=None):
         path = os.path.normpath(os.path.abspath(os.path.join(where, path)))
     if path is None or not os.path.exists(path):
         if where != os.getcwd():
-            # try the cwd instead
-            path = os.path.normpath(os.path.abspath(os.path.join(os.getcwd(),
-                                                                 orig)))
+            path = os.path.normpath(
+                os.path.abspath(os.path.join(os.getcwd(), orig))
+            )
     if path is None or not os.path.exists(path):
         return None
     if os.path.isdir(path):
-        # might want an __init__.py from pacakge
-        init = os.path.join(path,'__init__.py')
+        init = os.path.join(path, '__init__.py')
         if os.path.isfile(init):
             return init
     elif os.path.isfile(path):
@@ -125,17 +113,13 @@ def absfile(path, where=None):
 
 
 def anyp(predicate, iterable):
-    for item in iterable:
-        if predicate(item):
-            return True
-    return False
+    return any(predicate(item) for item in iterable)
 
 
 def file_like(name):
     """A name is file-like if it is a path that exists, or it has a
     directory part, or it ends in .py, or it isn't a legal python
-    identifier.
-    """
+    identifier."""
     return (os.path.exists(name)
             or os.path.dirname(name)
             or name.endswith('.py')
@@ -144,8 +128,7 @@ def file_like(name):
 
 def func_lineno(func):
     """Get the line number of a function. First looks for
-    compat_co_firstlineno, then func_code.co_first_lineno.
-    """
+    compat_co_firstlineno, then func_code.co_first_lineno."""
     try:
         return func.compat_co_firstlineno
     except AttributeError:
@@ -156,21 +139,15 @@ def func_lineno(func):
 
 
 def isclass(obj):
-    """Is obj a class? Inspect's isclass is too liberal and returns True
-    for objects that can't be subclasses of anything.
-    """
     obj_type = type(obj)
     return obj_type in class_types or issubclass(obj_type, type)
 
 
-# backwards compat (issue #64)
 is_generator = isgenerator
 
 
 def ispackage(path):
-    """
-    Is this path a package directory?
-
+    """Is this path a package directory?
     >>> ispackage('nose')
     True
     >>> ispackage('unit_tests')
@@ -178,8 +155,7 @@ def ispackage(path):
     >>> ispackage('nose/plugins')
     True
     >>> ispackage('nose/loader.py')
-    False
-    """
+    False """
     if os.path.isdir(path):
         # at least the end of the path must be a legal python identifier
         # and __init__.py[co] must exist
@@ -195,43 +171,37 @@ def ispackage(path):
 
 
 def isproperty(obj):
-    """
-    Is this a property?
-
+    """Is this a property?
     >>> class Foo:
     ...     def got(self):
     ...         return 2
     ...     def get(self):
     ...         return 1
     ...     get = property(get)
-
     >>> isproperty(Foo.got)
     False
     >>> isproperty(Foo.get)
-    True
-    """
-    return type(obj) == property
+    True """
+    return type(obj) is property
 
 
 def getfilename(package, relativeTo=None):
     """Find the python source file for a package, relative to a
     particular directory (defaults to current working directory if not
-    given).
-    """
+    given)."""
     if relativeTo is None:
         relativeTo = os.getcwd()
     path = os.path.join(relativeTo, os.sep.join(package.split('.')))
-    suffixes = ('/__init__.py', '.py')
-    for suffix in suffixes:
-        filename = path + suffix
-        if os.path.exists(filename):
-            return filename
+    if os.path.exists(path + '/__init__.py'):
+        return path
+    filename = path + '.py'
+    if os.path.exists(filename):
+        return filename
     return None
 
 
 def getpackage(filename):
-    """
-    Find the full dotted package name for a given python source file
+    """Find the full dotted package name for a given python source file
     name. Returns None if the file is not a python source file.
 
     >>> getpackage('foo.py')
@@ -257,10 +227,12 @@ def getpackage(filename):
 
     >>> path = os.path.abspath(os.path.join('nose', 'plugins'))
     >>> getpackage(path)
-    'nose.plugins'
-    """
+    'nose.plugins' """
     src_file = src(filename)
-    if (os.path.isdir(src_file) or not src_file.endswith('.py')) and not ispackage(src_file):
+    if (
+        os.path.isdir(src_file)
+        or not src_file.endswith('.py')
+    ) and not ispackage(src_file):
         return None
     base, ext = os.path.splitext(os.path.basename(src_file))
     if base == '__init__':
@@ -280,7 +252,6 @@ def getpackage(filename):
 
 def ln(label):
     """Draw a 70-char-wide divider, with label in the middle.
-
     >>> ln('hello there')
     '---------------------------- hello there -----------------------------'
     """
@@ -296,12 +267,10 @@ def ln(label):
 def resolve_name(name, module=None):
     """Resolve a dotted name to a module and its parts. This is stolen
     wholesale from unittest.TestLoader.loadTestByName.
-
     >>> resolve_name('nose.util') #doctest: +ELLIPSIS
     <module 'nose.util' from...>
     >>> resolve_name('nose.util.resolve_name') #doctest: +ELLIPSIS
-    <function resolve_name at...>
-    """
+    <function resolve_name at...> """
     parts = name.split('.')
     parts_copy = parts[:]
     if module is None:
@@ -325,49 +294,33 @@ def resolve_name(name, module=None):
 def split_test_name(test):
     """Split a test name into a 3-tuple containing file, module, and callable
     names, any of which (but not all) may be blank.
-
     Test names are in the form:
-
     file_or_module:callable
-
     Either side of the : may be dotted. To change the splitting behavior, you
-    can alter nose.util.split_test_re.
-    """
+    can alter nose.util.split_test_re."""
     norm = os.path.normpath
     file_or_mod = test
     fn = None
-    if not ':' in test:
-        # only a file or mod part
+    if ':' not in test:
         if file_like(test):
             return (norm(test), None, None)
         else:
             return (None, test, None)
-
-    # could be path|mod:callable, or a : in the file path someplace
     head, tail = os.path.split(test)
     if not head:
-        # this is a case like 'foo:bar' -- generally a module
-        # name followed by a callable, but also may be a windows
-        # drive letter followed by a path
         try:
             file_or_mod, fn = test.split(':')
             if file_like(fn):
-                # must be a funny path
                 file_or_mod, fn = test, None
         except ValueError:
-            # more than one : in the test
-            # this is a case like c:\some\path.py:a_test
             parts = test.split(':')
             if len(parts[0]) == 1:
                 file_or_mod, fn = ':'.join(parts[:-1]), parts[-1]
             else:
-                # nonsense like foo:bar:baz
                 raise ValueError("Test name '%s' could not be parsed. Please "
                                  "format test names as path:callable or "
                                  "module:callable." % (test,))
     elif not tail:
-        # this is a case like 'foo:bar/'
-        # : must be part of the file path, so ignore it
         file_or_mod = test
     else:
         if ':' in tail:
@@ -382,24 +335,27 @@ def split_test_name(test):
             return (None, file_or_mod, fn)
     else:
         return (None, None, fn)
-split_test_name.__test__ = False # do not collect
+
+
+split_test_name.__test__ = False  # do not collect
 
 
 def test_address(test):
     """Find the test address for a test, which may be a module, filename,
-    class, method or function.
-    """
+    class, method or function."""
     if hasattr(test, "address"):
         return test.address()
-    # type-based polymorphism sucks in general, but I believe is
-    # appropriate here
     t = type(test)
     file = module = call = None
-    if t == types.ModuleType:
+    if t == types.ModuleType:  # noqa: E721
         file = getattr(test, '__file__', None)
         module = getattr(test, '__name__', None)
         return (src(file), module, call)
-    if t == types.FunctionType or issubclass(t, type) or t == type:
+    if (
+        t == types.FunctionType  # noqa: E721
+        or issubclass(t, type)
+        or t == type
+    ):
         module = getattr(test, '__module__', None)
         if module is not None:
             m = sys.modules[module]
@@ -408,22 +364,22 @@ def test_address(test):
                 file = os.path.abspath(file)
         call = getattr(test, '__name__', None)
         return (src(file), module, call)
-    if t == types.MethodType:
+    if t == types.MethodType:  # noqa: E721
         cls_adr = test_address(test.__self__.__class__)
         return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], test.__name__))
     # handle unittest.TestCase instances
     if isinstance(test, unittest.TestCase):
-        if (hasattr(test, '_FunctionTestCase__testFunc') # pre 2.7
-            or hasattr(test, '_testFunc')):              # 2.7
-            # unittest FunctionTestCase
+        if (
+            hasattr(test, '_FunctionTestCase__testFunc')  # pre 2.7
+            or hasattr(test, '_testFunc')  # 2.7 and newer
+        ):
             try:
                 return test_address(test._FunctionTestCase__testFunc)
             except AttributeError:
                 return test_address(test._testFunc)
         # regular unittest.TestCase
         cls_adr = test_address(test.__class__)
-        # 2.5 compat: __testMethodName changed to _testMethodName
         try:
             method_name = test._TestCase__testMethodName
         except AttributeError:
@@ -434,30 +390,34 @@ def test_address(test):
             test.__class__.__module__ not in ('__builtin__', 'builtins')):
         return test_address(test.__class__)
     raise TypeError("I don't know what %s is (%s)" % (test, t))
-test_address.__test__ = False # do not collect
+
+
+test_address.__test__ = False  # do not collect
 
 
 def try_run(obj, names):
     """Given a list of possible method names, try to run them with the
     provided object. Keep going until something works. Used to run
-    setup/teardown methods for module, package, and function tests.
-    """
+    setup/teardown methods for module, package, and function tests."""
     for name in names:
         func = getattr(obj, name, None)
         if func is not None:
-            if type(obj) == types.ModuleType:
+            if isinstance(obj, types.ModuleType):
                 # py.test compatibility
                 if isinstance(func, types.FunctionType):
-                    args, varargs, varkw, defaults = \
-                        inspect.getargspec(func)
+                    args, varargs, varkw, defaults = inspect.getargspec(func)
                 else:
-                    # Not a function. If it's callable, call it anyway
-                    if hasattr(func, '__call__') and not inspect.ismethod(func):
+                    # Not a function. If it's callable, call it anyway.
+                    if (
+                        callable(func)
+                        and not inspect.ismethod(func)
+                    ):
                         func = func.__call__
                     try:
-                        args, varargs, varkw, defaults = \
+                        args, varargs, varkw, defaults = (
                             inspect.getargspec(func)
-                        args.pop(0) # pop the self off
+                        )
+                        args.pop(0)  # pop the self off
                     except TypeError:
                         raise TypeError("Attribute %s of %r is not a python "
                                         "function. Only functions or callables"
@@ -468,13 +428,13 @@ def try_run(obj, names):
                     return func(obj)
             log.debug("call fixture %s.%s", obj, name)
             return func()
+    return None
 
 
 def src(filename):
     """Find the python source file for a .pyc, .pyo or $py.class file on
     jython. Returns the filename provided if it is not a python source
-    file.
-    """
+    file."""
     if filename is None:
         return filename
     if sys.platform.startswith('java') and filename.endswith('$py.class'):
@@ -488,7 +448,6 @@ def src(filename):
 def regex_last_key(regex):
     """Sort key function factory that puts items that match a
     regular expression last.
-
     >>> from nose.config import Config
     >>> from nose.pyversion import sort_list
     >>> c = Config()
@@ -496,8 +455,7 @@ def regex_last_key(regex):
     >>> entries = ['.', '..', 'a_test', 'src', 'lib', 'test', 'foo.py']
     >>> sort_list(entries, regex_last_key(regex))
     >>> entries
-    ['.', '..', 'foo.py', 'lib', 'src', 'a_test', 'test']
-    """
+    ['.', '..', 'foo.py', 'lib', 'src', 'a_test', 'test'] """
     def k(obj):
         if regex.search(obj):
             return (1, obj)
@@ -508,35 +466,28 @@ def regex_last_key(regex):
 def tolist(val):
     """Convert a value that may be a list or a (possibly comma-separated)
     string into a list. The exception: None is returned as None, not [None].
-
     >>> tolist(["one", "two"])
     ['one', 'two']
     >>> tolist("hello")
     ['hello']
     >>> tolist("separate,values, with, commas,  spaces , are    ,ok")
-    ['separate', 'values', 'with', 'commas', 'spaces', 'are', 'ok']
-    """
+    ['separate', 'values', 'with', 'commas', 'spaces', 'are', 'ok'] """
     if val is None:
         return None
     try:
-        # might already be a list
         val.extend([])
         return val
     except AttributeError:
         pass
-    # might be a string
     try:
         return re.split(r'\s*,\s*', val)
     except TypeError:
-        # who knows...
         return list(val)
 
 
 class odict(dict):
     """Simple ordered dict implementation, based on:
-
-    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/107747
-    """
+    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/107747 """
     def __init__(self, *arg, **kw):
         self._keys = []
         super(odict, self).__init__(*arg, **kw)
@@ -551,7 +502,9 @@ class odict(dict):
             self._keys.append(key)
 
     def __str__(self):
-        return "{%s}" % ', '.join(["%r: %r" % (k, v) for k, v in list(self.items())])
+        return "{%s}" % ', '.join(
+            ["%r: %r" % (k, v) for k, v in list(self.items())]
+        )
 
     def clear(self):
         super(odict, self).clear()
@@ -585,10 +538,8 @@ class odict(dict):
 
 
 def transplant_func(func, module):
-    """
-    Make a function imported from module A appear as if it is located
+    """Make a function imported from module A appear as if it is located
     in module B.
-
     >>> from pprint import pprint
     >>> pprint.__module__
     'pprint'
@@ -597,23 +548,19 @@ def transplant_func(func, module):
     'nose.util'
 
     The original function is not modified.
-
     >>> pprint.__module__
     'pprint'
 
     Calling the transplanted function calls the original.
-
     >>> pp([1, 2])
     [1, 2]
     >>> pprint([1,2])
-    [1, 2]
-
-    """
+    [1, 2] """
     from nose.tools import make_decorator
+
     if isgenerator(func):
         def newfunc(*arg, **kw):
-            for v in func(*arg, **kw):
-                yield v
+            yield from func(*arg, **kw)
     else:
         def newfunc(*arg, **kw):
             return func(*arg, **kw)
@@ -624,10 +571,8 @@ def transplant_func(func, module):
 
 
 def transplant_class(cls, module):
-    """
-    Make a class appear to reside in `module`, rather than the module in which
-    it is actually defined.
-
+    """Make a class appear to reside in `module`,
+    rather than the module in which it is actually defined.
     >>> from nose.failure import Failure
     >>> Failure.__module__
     'nose.failure'
@@ -635,11 +580,10 @@ def transplant_class(cls, module):
     >>> Nf.__module__
     'nose.util'
     >>> Nf.__name__
-    'Failure'
-
-    """
+    'Failure' """
     class C(cls):
         pass
+
     C.__module__ = module
     C.__name__ = cls.__name__
     return C
@@ -650,11 +594,18 @@ def safe_str(val, encoding='utf-8'):
         return str(val)
     except UnicodeEncodeError:
         if isinstance(val, Exception):
-            return ' '.join([safe_str(arg, encoding)
-                             for arg in val])
+            return ' '.join([safe_str(arg, encoding) for arg in val])
         return str(val).encode(encoding)
 
-    
+
+def is_executable(file):
+    if not os.path.exists(file):
+        return False
+    st = os.stat(file)
+    return bool(st.st_mode & (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH))
+
+
 if __name__ == '__main__':
     import doctest
+
     doctest.testmod()
